package net.h34t.filemure

import io.javalin.http.ContentType
import io.javalin.http.Context
import io.javalin.http.HttpStatus
import io.javalin.http.UnauthorizedResponse
import java.io.BufferedOutputStream
import java.io.OutputStreamWriter
import java.time.Month

/**
 * Outputs an HTML page rendered via TempolinTemplate.
 *
 * This uses a buffered OutputStreamWriter, so the rendered output can be written partially during creation.
 *
 * @param tpl The Tempolin Template to render
 * @param contentType An optional content type (defaults to HTML)
 */
fun Context.tempolin(tpl: Template, contentType: String = ContentType.HTML) {
    this.contentType(contentType)
    return OutputStreamWriter(BufferedOutputStream(this.outputStream())).use { writer ->
        tpl.writeTo(writer)
    }
}

/**
 * A redirect using SEE_OTHER according to the post-redirect-get pattern
 * see https://en.wikipedia.org/wiki/Post/Redirect/Get
 *
 * @param location The location to redirect to.
 *
 */
fun Context.redirectPRG(location: String) = this.redirect(location, HttpStatus.SEE_OTHER)

fun Context.getSession() = this.sessionAttribute<Session>("session")
fun Context.setSession(session: Session?) = this.sessionAttribute("session", session)

fun Context.requireSession(): Session = this.getSession() ?: throw UnauthorizedResponse("Not logged in")

fun formatHumanReadableSize(bytes: Long) = when (bytes) {
    in 0L..<1024L -> "$bytes bytes"
    in 1025..<1024 * 1_000 -> "${bytes / 1000} kb"
    in 1025 * 1_000..<1024 * 1_000_000 -> "${bytes / 1_000_000} mb"
    else -> "${bytes / 1_000_000_000} gb"
}

object TagAdapter {
    fun parse(ser: String?): List<Tag> {
        return ser?.let { if (it.isNotBlank()) it.split(",").map { Tag(it) } else emptyList() } ?: emptyList()
    }

    fun List<Tag>.serialize() = if (this.isEmpty()) "" else this.joinToString(",") { it.value }
}

fun List<Document>.grouped(): Map<Int, Map<Month, List<Document>>> =
    this.groupBy { it.referenceDate.year }
        .mapValues { (_, values) ->
            values.groupBy { it.referenceDate.month }
        }

